import gnu.io.PortInUseException;
import gnu.io.UnsupportedCommOperationException;

import java.awt.Toolkit;
import java.io.IOException;
import java.util.TooManyListenersException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import pic113.Pic113;
import pic113.Pic113.Pic113Commands;

import comport.ComPortNotFoundException;

//Todo
//1) Write a callback function for pic113, to interpret the data receive from the com port
//2) To activate a customised output or read input by running the *.jar file 

public class MyMain
{
	protected static transient final Log logger = LogFactory.getLog(MyMain.class);

	public static void main(String[] args)
	{		
		String comName = null;
		if(args.length==0)
			comName = null;
		else
			comName = args[0];
		
		try
		{
			if(args.length<=1)		//run demo
			{
				demo(comName);
			}
			else if(args.length==2)	//control output 
			{
				//int portNum = Integer.parseInt(args[1]);
				//int onOff = Integer.parseInt(args[2]);
			}
		}
		catch(PortInUseException e)
		{
			logger.error("Serial Com port is in use. Please ensure that the application close the port to release the port.");
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		catch(UnsupportedCommOperationException e)
		{
			e.printStackTrace();
		}
		catch(ComPortNotFoundException e)
		{
			logger.error("ComPortNotFoundException, ", e);
		}
		catch(TooManyListenersException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	static void demo(String comName) throws ComPortNotFoundException, PortInUseException, IOException, UnsupportedCommOperationException, TooManyListenersException
	{
		logger.info("****************************************************************");
		logger.info("PIC-113 USB I/O Controller DEMO program.");
		logger.info("Each steop takes about 5sec. Program duration is about 90sec.");
		logger.info("You can triggered input during the demo.");
		logger.info("Input notification message will be displayed immediately.");
		logger.info("****************************************************************");
		
		Pic113 pic = new Pic113(comName);

		logger.info("Program started. Indicator at Port3 & Port4 should be off.");
		pic.sendCommand(Pic113Commands.OUT3_OFF);
		pic.sendCommand(Pic113Commands.OUT4_OFF);
		delay();
		

		logger.info("Port3 output demo will start in another 5sec.");
		delay();

		logger.info("Turn on Port3 output. You should see the indicator light up.");
		pic.sendCommand(Pic113Commands.OUT3_ON);
		delay();

		logger.info("Turn off Port3 output.");
		pic.sendCommand(Pic113Commands.OUT3_OFF);
		delay();

		logger.info("+ve Pulse Port3 output. You should see the indicator light up for about 1sec before it restore to ON.");
		pic.sendCommand(Pic113Commands.OUT3_POSPULSE);
		delay();

		logger.info("Toggle Port3 output. You should see the indicator change state.");
		pic.sendCommand(Pic113Commands.OUT3_TOGGLE);
		delay();

		logger.info("-ve Pulse Port3 output. You should see the indicator light off for about 1sec before it restore to ON.");
		pic.sendCommand(Pic113Commands.OUT3_NEGPULSE);
		delay();

		logger.info("Turn off Port3 output.");
		pic.sendCommand(Pic113Commands.OUT3_OFF);
		delay();

		
		logger.info("Port4 output demo will start in another 5sec.");
		delay();

		logger.info("Turn on Port4 output. You should see the indicator light up.");
		pic.sendCommand(Pic113Commands.OUT4_ON);
		delay();

		logger.info("Turn off Port4 output.");
		pic.sendCommand(Pic113Commands.OUT4_OFF);
		delay();
		
		logger.info("+ve Pulse Port4 output. You should see the indicator light up for about 1sec before it restore to ON.");
		pic.sendCommand(Pic113Commands.OUT4_POSPULSE);
		delay();

		logger.info("Toggle Port4 output. You should see the indicator change state.");
		pic.sendCommand(Pic113Commands.OUT4_TOGGLE);
		delay();

		logger.info("-ve Pulse Port4 output. You should see the indicator light off for about 1sec before it restore to ON.");
		pic.sendCommand(Pic113Commands.OUT4_NEGPULSE);
		delay();

		logger.info("Turn off Port4 output.");
		pic.sendCommand(Pic113Commands.OUT4_OFF);
		delay();

		logger.info("Now try trigger the input with a simple switch.");
		logger.info("Message will be send when the input is triggered.");
		logger.info("You have 10sec before the DEMO end.");
		delay();
		delay();
		
		logger.info("End of DEMO.");
		pic.close();
	}
	
	static public void delay()
	{
		Toolkit.getDefaultToolkit().beep(); //error beep tone (sound, bell) 
		
		try
		{
			Thread.sleep(5000); // Be sure data is xferred before closing
		}
		catch(Exception e)
		{
		}
	}
}
