package comport;

import gnu.io.CommPortIdentifier;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;
import gnu.io.UnsupportedCommOperationException;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.TooManyListenersException;

import javax.xml.bind.annotation.adapters.HexBinaryAdapter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import utilities.HexUtilities;

//2012-07-19 Installing the library for serial communication, Java Communication API (RXTXcomm.jar).
//download RXTXcomm for 64bit or 32bit system
//copy RXTXcomm.jar to your java project\lib folder, and import into your project
//copy rxtxSerial.dll to System32 (for 64bits JVM), copy to SysWOW64 (for 32bits JVM)
//The following error will occur if the rxtxSerial.dll is copied to the wrong place.
//java.lang.UnsatisfiedLinkError: no rxtxSerial in java.library.path thrown while loading gnu.io.RXTXCommDriver
//Exception in thread "main" java.lang.UnsatisfiedLinkError: no rxtxSerial in java.library.path
//	at java.lang.ClassLoader.loadLibrary(Unknown Source)
//	at java.lang.Runtime.loadLibrary0(Unknown Source)
//	at java.lang.System.loadLibrary(Unknown Source)
//	at gnu.io.CommPortIdentifier.<clinit>(CommPortIdentifier.java:83)
//	at ComPort.main(ComPort.java:34)
//The following error will occur if the wrong rxtxSerial.dll version (32bit or 64bit) is used with the JVM
//java.lang.UnsatisfiedLinkError: C:\Windows\System32\rxtxSerial.dll: Can't load IA 32-bit .dll on a AMD 64-bit platform thrown while loading gnu.io.RXTXCommDriver
//Exception in thread "main" java.lang.UnsatisfiedLinkError: C:\Windows\System32\rxtxSerial.dll: Can't load IA 32-bit .dll on a AMD 64-bit platform
//	at java.lang.ClassLoader$NativeLibrary.load(Native Method)
//	at java.lang.ClassLoader.loadLibrary1(Unknown Source)
//	at java.lang.ClassLoader.loadLibrary0(Unknown Source)
//	at java.lang.ClassLoader.loadLibrary(Unknown Source)
//	at java.lang.Runtime.loadLibrary0(Unknown Source)
//	at java.lang.System.loadLibrary(Unknown Source)
//	at gnu.io.CommPortIdentifier.<clinit>(CommPortIdentifier.java:83)
//	at ComPort.main(ComPort.java:44)

//import gnu.io.* in your java source code to start using the classes in the API

public class SerialComPortMgr implements SerialPortEventListener, Runnable
{
	protected static transient final Log logger = LogFactory.getLog(SerialComPortMgr.class);

	private static SerialComPortMgr scpMgr = null;

	private String appsName;

	private static final int openport_timeout = 2000;

	//	private CommPort port;
	private static CommPortIdentifier selectedPortId;
	private static SerialPort serialPort;
	private static OutputStream outputStream;
	private static InputStream inputStream;
	
	private static Thread		readThread;

	public static SerialComPortMgr getInstance(String appsName)
	{ //object can only create once. constructor method is private
		if(scpMgr == null)
			scpMgr = new SerialComPortMgr(appsName);
		return(scpMgr);
	}

	private SerialComPortMgr(String appsName)
	{
		this.appsName = appsName;
	}

	public void open(String comPortName) throws ComPortNotFoundException, PortInUseException, IOException, TooManyListenersException
	{
		logger.info("Opening Com Port...");

		//retrieve a list of com port available
		Enumeration<CommPortIdentifier> portList = CommPortIdentifier.getPortIdentifiers();

		selectedPortId = null;

		while(portList.hasMoreElements())
		{
			CommPortIdentifier portId = portList.nextElement();
			logger.info(" found->" + portId.getName());

			//check if port is a serial port
			if(portId.getPortType() == CommPortIdentifier.PORT_SERIAL)
			{
				if(comPortName == null) //no port name specified, therefore use the very first com port that comes along
				{
					selectedPortId = portId;
					break;
				}
				else if(comPortName.compareToIgnoreCase(portId.getName()) == 0)
				{
					selectedPortId = portId;
					break;
				}
			}
		}

		if(selectedPortId == null) //
		{
			if(comPortName == null)
				throw new ComPortNotFoundException("No Com Port is found on the system.");
			else
				throw new ComPortNotFoundException("Cannot find Com Port \"" + comPortName + "\"");
		}

		comPortName = selectedPortId.getName(); //update the name of the selected com port
		logger.info("Opening Com Port \"" + comPortName + "\".");

		serialPort = (SerialPort) selectedPortId.open(appsName, openport_timeout);
		outputStream = serialPort.getOutputStream(); //get stream for output
		
		inputStream = serialPort.getInputStream(); //get stream for input		
		serialPort.notifyOnDataAvailable(true);
		serialPort.addEventListener(this); //register listener for incoming data.
		logger.info("Com Port \"" + comPortName + "\" opened.");
		
		readThread = new Thread(this);
		readThread.start();
	}

	public void setComPort(int baudrate, int data, int stop, int parity) throws UnsupportedCommOperationException
	{
		serialPort.setSerialPortParams(9600, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
	}

	public void close()
	{
		logger.info("Closing Com Port \"" + selectedPortId.getName() + "\".");
		serialPort.close();
	}

	public void send(byte[] byteArray)
	{
		try
		{
			outputStream.write(byteArray);
			String tx = HexUtilities.hexToHexString(byteArray, byteArray.length, ' ');
			logger.debug(byteArray.length + "bytes sent    ->" + tx);
		}
		catch(IOException e)
		{
			logger.error("error sending bytes \"" + byteArray + "\"");
		}
	}

	public void run()
	{
		try
		{
			Thread.sleep(20000);
		}
		catch(InterruptedException e)
		{
		}
	}

	@Override
	public void serialEvent(SerialPortEvent event)
	{
		if(event.getEventType() == SerialPortEvent.BI)
		{
		}
		else if(event.getEventType() == SerialPortEvent.OE)
		{
		}
		else if(event.getEventType() == SerialPortEvent.FE)
		{
		}
		else if(event.getEventType() == SerialPortEvent.PE)
		{
		}
		else if(event.getEventType() == SerialPortEvent.CD)
		{
		}
		else if(event.getEventType() == SerialPortEvent.CTS)
		{
		}
		else if(event.getEventType() == SerialPortEvent.DSR)
		{
		}
		else if(event.getEventType() == SerialPortEvent.RI)
		{
		}
		else if(event.getEventType() == SerialPortEvent.OUTPUT_BUFFER_EMPTY)
		{
		}
		else if(event.getEventType() == SerialPortEvent.DATA_AVAILABLE)
		{
			byte[] readBuffer = new byte[20];

			try
			{
				int numBytes = 0;
				while(inputStream.available() > 0)
				{
					numBytes = inputStream.read(readBuffer);
				}
				
				String rx = HexUtilities.hexToHexString(readBuffer, numBytes, ' ');
				logger.debug(numBytes +         "bytes received<-" + rx);
				
				{//to be rewritten as call back function to Pic113
					if(rx.compareToIgnoreCase("03 49 50 30 31 30 30 0D 0A")==0)
						logger.info("Input 1 Off");
					else if(rx.compareToIgnoreCase("03 49 50 30 31 30 31 0D 0A")==0)
						logger.info("Input 1 On");
					else if(rx.compareToIgnoreCase("03 49 50 30 32 30 30 0D 0A")==0)
						logger.info("Input 2 Off");
					else if(rx.compareToIgnoreCase("03 49 50 30 32 30 31 0D 0A")==0)
						logger.info("Input 2 On");
				}
			}
			catch(IOException e)
			{
			}
		}
	}
}
