package pic113;

import java.io.IOException;
import java.util.TooManyListenersException;

import comport.ComPortNotFoundException;
import comport.SerialComPortMgr;

import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;
import utilities.shutdownHook.ShutdownHook;
import utilities.shutdownHook.ShutdownHookClass;

public class Pic113 implements ShutdownHookClass
{
	private static Pic113 pic;
	@SuppressWarnings("unused")
	private ShutdownHook sdh = null; //for shutdownhook
    
    private SerialComPortMgr scpMgr = null;
	
	public Pic113 getInstance(String comPort) throws ComPortNotFoundException, PortInUseException, IOException, UnsupportedCommOperationException, TooManyListenersException
    { //object can only create once. constructor method is private
        if(pic == null)           
        	pic = new Pic113(comPort);
        return(pic);
    }
	
	public Pic113(String comPort) throws ComPortNotFoundException, PortInUseException, IOException, UnsupportedCommOperationException, TooManyListenersException
	{
		//comport is the prefered name of the com port to open
		scpMgr = SerialComPortMgr.getInstance("PIC-113 USB I/O Controller");
		scpMgr.open(comPort);
		scpMgr.setComPort(9600, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);

	}
	
	public void open()
	{
		
	}

	public void close()
	{
		scpMgr.close();
	}

	public void sendCommand(Pic113Commands cmd)
	{
		scpMgr.send(cmd.getBytes());
	}
	
	public enum Pic113Commands
	{
		OK(new byte[] {0x03,'O','K','\r','\n'}), 					ESC(new byte[] {0x1B}),
		IN1_READ(new byte[]{0x03,'I','N','0','1','\r','\n'}), 		IN2_READ(new byte[]{0x03,'I','N','0','2','\r','\n'}),
		OUT3_TOGGLE(new byte[]{0x03,'O','P','0','3','\r','\n'}), 	OUT4_TOGGLE(new byte[]{0x03,'O','P','0','4','\r','\n'}),
		OUT3_OFF(new byte[]{0x03,'O','P','0','3','0','0','\r','\n'}), OUT4_OFF(new byte[]{0x03,'O','P','0','4','0','0','\r','\n'}),
		OUT3_ON(new byte[]{0x03,'O','P','0','3','0','1','\r','\n'}), OUT4_ON(new byte[]{0x03,'O','P','0','4','0','1','\r','\n'}),
		OUT3_POSPULSE(new byte[]{0x03,'O','1','0','3','\r','\n'}), 	OUT4_POSPULSE(new byte[]{0x03,'O','1','0','4','\r','\n'}),
		OUT3_NEGPULSE(new byte[]{0x03,'O','2','0','3','\r','\n'}), 	OUT4_NEGPULSE(new byte[]{0x03,'O','2','0','4','\r','\n'}),
		OUT3_TOGPULSE(new byte[]{0x03,'O','3','0','3','\r','\n'}), 	OUT4_TOGPULSE(new byte[]{0x03,'O','3','0','4','\r','\n'}),
		OFF_INPUTCHANGENOTIFICATION(new byte[]{0x03,'S','C','I','C','0','0','\r','\n'}),
		ON_INPUTCHANGENOTIFICATION(new byte[]{0x03,'S','C','I','C','0','1','\r','\n'}),
		GET_INPUTCHANGENOTIFICATION(new byte[]{0x03,'G','C','I','C','\r','\n'});

		//constructor cannot be public
		//constructor allows multiple parameters, so that
		//the enum can represent more than 1 parameter.
		private Pic113Commands(byte[] cmd)
		{
			this.cmd = cmd;
		}
	   
		private byte[] cmd = null;
		public byte[] getBytes()
		{
			return cmd;
		}
	}

	@Override
	public void shutdownHookCallback()
	{	
		close();	//close if there is any abnormal program termination		
	}
}
