package utilities;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ConfigFile
{
	protected static transient final Log logger = LogFactory.getLog(ConfigFile.class);

	private Properties myConfig = null;		//map property for group address mapping 
	private String filename = "/config.properties";

	public ConfigFile(String filename)
    {
		this.filename = filename; 
    	loadConfig();
    }
    
    //Example of reading configuration from a property file
	public void loadConfig()
	{
	    InputStream is = getClass().getResourceAsStream(filename);	//for files under the project folder /resource 
	    myConfig = new Properties();

	    try
	    {
	        myConfig.load(is);
	        is.close();
	    }
	    catch (IOException ioe)
	    {   //ensure that the file is in the classpath");
	        logger.error("Error reading the properties file.");       
	        System.exit(1);
	    }
	}
	
	public String getProperty(String key)
	{
	    String str;
	    str = myConfig.getProperty(key, null);
	    if(str!=null)
	    {
		    str = str.trim();
		    if((str.compareToIgnoreCase("null")==0)||(str.compareToIgnoreCase("")==0))
		    		return(null);
	    }
	    return(str);
	}
	
	public Integer getPropertyInteger(String key)
	{
	    String str = getProperty(key).trim();
	    int i = 0;
	    if(str != null)
		{
			try
			{
				i = Integer.parseInt(str);
			}
			catch(NumberFormatException e)
			{
				logger.error("NumberFormatException occured in property file, " + key + "=" + str);
				logger.info("An example for the configuration, " + key + "=100");
				return(null);
			}
		}
	    return(i);
	}

	public static void main(String[] args)
	{
		@SuppressWarnings("unused")
		ConfigFile c = new ConfigFile("/config.properties");
	}
}
